"""
Tests for the vSQL left shift operator ``<<``.

The test are done via the Python DB interface.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

def test_bool_bool1(config_data):
	check_vsql(config_data, "app.p_int_none.value << False is None")

def test_bool_bool2(config_data):
	check_vsql(config_data, "app.p_bool_false.value << True == 0")

def test_bool_bool3(config_data):
	check_vsql(config_data, "app.p_bool_true.value << True == 2")

def test_bool_int(config_data):
	check_vsql(config_data, "app.p_bool_true.value << 1 == 2")

def test_int_bool(config_data):
	check_vsql(config_data, "2 << app.p_bool_true.value == 4")

def test_int_int(config_data):
	check_vsql(config_data, "-app.p_int_value.value << 2 == -7108")



"""
Tests for the vSQL multiplication operator ``*``.

The test are done via the Python DB interface.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

def test_bool_bool1(config_data):
	check_vsql(config_data, "app.p_int_none.value * True is None")

def test_bool_bool2(config_data):
	check_vsql(config_data, "app.p_bool_false.value * True == 0")

def test_bool_bool3(config_data):
	check_vsql(config_data, "app.p_bool_true.value * True == 1")

def test_bool_int(config_data):
	check_vsql(config_data, "app.p_bool_true.value * 1 == 1")

def test_bool_number(config_data):
	check_vsql(config_data, "app.p_bool_true.value * 1.5 == 1.5")

def test_int_bool(config_data):
	check_vsql(config_data, "2 * app.p_bool_true.value == 2")

def test_int_int(config_data):
	check_vsql(config_data, "2 *  app.p_int_value.value == 3554")

def test_int_number(config_data):
	check_vsql(config_data, "2 * app.p_number_value.value == 85.0")

def test_number_bool(config_data):
	check_vsql(config_data, "app.p_number_value.value * app.p_bool_true.value == 42.5")

def test_number_int(config_data):
	check_vsql(config_data, "app.p_number_value.value * 2 == 85.0")

def test_number_number(config_data):
	check_vsql(config_data, "app.p_number_value.value * 1.5 == 63.75")

def test_bool_str1(config_data):
	check_vsql(config_data, "app.p_bool_none.value * app.p_str_none.value == ''")

def test_bool_str2(config_data):
	check_vsql(config_data, "app.p_bool_false.value * app.p_str_none.value == ''")

def test_bool_str3(config_data):
	check_vsql(config_data, "app.p_bool_true.value * app.p_str_none.value == ''")

def test_bool_str4(config_data):
	check_vsql(config_data, "app.p_bool_none.value * app.p_str_value.value == ''")

def test_bool_str5(config_data):
	check_vsql(config_data, "app.p_bool_false.value * app.p_str_value.value == ''")

def test_bool_str6(config_data):
	check_vsql(config_data, "app.p_bool_true.value * app.p_str_value.value == 'gurk'")

def test_int_str1(config_data):
	check_vsql(config_data, "app.p_int_none.value * app.p_str_none.value == ''")

def test_int_str2(config_data):
	check_vsql(config_data, "2 * app.p_str_none.value == ''")

def test_int_str3(config_data):
	check_vsql(config_data, "app.p_int_none.value * app.p_str_value.value == ''")

def test_int_str4(config_data):
	check_vsql(config_data, "2 * app.p_str_value.value == 'gurkgurk'")

def test_bool_datedelta1(config_data):
	check_vsql(config_data, "app.p_bool_none.value * days(3) is None")

def test_bool_datedelta2(config_data):
	check_vsql(config_data, "True * app.p_datedelta_none.value is None")

def test_bool_datedelta3(config_data):
	check_vsql(config_data, "True * app.p_datedelta_value.value == days(12)")

def test_int_datedelta1(config_data):
	check_vsql(config_data, "app.p_int_none.value * days(3) is None")

def test_int_datedelta2(config_data):
	check_vsql(config_data, "2 * app.p_datedelta_none.value is None")

def test_int_datedelta3(config_data):
	check_vsql(config_data, "2 * app.p_datedelta_value.value == days(24)")

def test_bool_datetimedelta1(config_data):
	check_vsql(config_data, "app.p_bool_none.value * minutes(3) is None")

def test_bool_datetimedelta2(config_data):
	check_vsql(config_data, "True * app.p_datetimedelta_none.value is None")

def test_bool_datetimedelta3(config_data):
	check_vsql(config_data, "True * app.p_datetimedelta_value.value == timedelta(1, (12 * 60 + 34) * 60 + 56)")

def test_int_datetimedelta1(config_data):
	check_vsql(config_data, "app.p_int_none.value * minutes(3) is None")

def test_int_datetimedelta2(config_data):
	check_vsql(config_data, "2 * app.p_datetimedelta_none.value is None")

def test_int_datetimedelta3(config_data):
	check_vsql(config_data, "2 * app.p_datetimedelta_value.value == timedelta(3, (1 * 60 + 9) * 60 + 52)")

def test_bool_monthdelta1(config_data):
	check_vsql(config_data, "app.p_bool_none.value * months(3) is None")

def test_bool_monthdelta2(config_data):
	check_vsql(config_data, "True * app.p_monthdelta_none.value is None")

def test_bool_monthdelta3(config_data):
	check_vsql(config_data, "True * app.p_monthdelta_value.value == months(3)")

def test_int_monthdelta1(config_data):
	check_vsql(config_data, "app.p_int_none.value * months(3) is None")

def test_int_monthdelta2(config_data):
	check_vsql(config_data, "2 * app.p_monthdelta_none.value is None")

def test_int_monthdelta3(config_data):
	check_vsql(config_data, "2 * app.p_monthdelta_value.value == months(6)")

def test_number_datetimedelta3(config_data):
	check_vsql(config_data, "2.5 * app.p_datetimedelta_value.value == timedelta(3, (19 * 60 + 27) * 60 + 20)")

def test_str_bool1(config_data):
	check_vsql(config_data, "app.p_str_none.value * app.p_bool_none.value == ''")

def test_str_bool2(config_data):
	check_vsql(config_data, "app.p_str_none.value * app.p_bool_false.value == ''")

def test_str_bool3(config_data):
	check_vsql(config_data, "app.p_str_none.value * app.p_bool_true.value == ''")

def test_str_bool4(config_data):
	check_vsql(config_data, "app.p_str_value.value * app.p_bool_none.value == ''")

def test_str_bool5(config_data):
	check_vsql(config_data, "app.p_str_value.value * app.p_bool_false.value == ''")

def test_str_bool6(config_data):
	check_vsql(config_data, "app.p_str_value.value * app.p_bool_true.value == 'gurk'")

def test_str_int1(config_data):
	check_vsql(config_data, "app.p_str_none.value * app.p_int_none.value == ''")

def test_str_int2(config_data):
	check_vsql(config_data, "app.p_str_none.value * 2 == ''")

def test_str_int3(config_data):
	check_vsql(config_data, "app.p_str_value.value * app.p_int_none.value == ''")

def test_bool_intlist1(config_data):
	check_vsql(config_data, "app.p_bool_none.value * [1, 2, 3] is None")

def test_bool_intlist2(config_data):
	check_vsql(config_data, "app.p_bool_false.value * [1, 2, 3] == [1][:0]")

def test_int_intlist1(config_data):
	check_vsql(config_data, "app.p_int_none.value * [1, 2, 3] is None")

def test_int_intlist2(config_data):
	check_vsql(config_data, "2 * [1, 2, 3] == [1, 2, 3, 1, 2, 3]")

def test_bool_nulllist1(config_data):
	check_vsql(config_data, "False * [] == []")

def test_bool_nulllist2(config_data):
	check_vsql(config_data, "True * [] == []")

def test_bool_nulllist3(config_data):
	check_vsql(config_data, "False * [None, None] == []")

def test_bool_nulllist4(config_data):
	check_vsql(config_data, "True * [None, None] == [None, None]")

def test_int_nulllist1(config_data):
	check_vsql(config_data, "0 * [] == []")

def test_int_nulllist2(config_data):
	check_vsql(config_data, "2 * [] == []")

def test_int_nulllist3(config_data):
	check_vsql(config_data, "0 * [None, None] == []")

def test_int_nulllist4(config_data):
	check_vsql(config_data, "2 * [None, None] == [None, None, None, None]")

def test_nulllist1_bool(config_data):
	check_vsql(config_data, "[] * False == []")

def test_nulllist2_bool(config_data):
	check_vsql(config_data, "[] * True == []")

def test_nulllist3_bool(config_data):
	check_vsql(config_data, "[None, None] * False == []")

def test_nulllist4_bool(config_data):
	check_vsql(config_data, "[None, None] * True == [None, None]")

def test_nulllist1_int(config_data):
	check_vsql(config_data, "[] * 0 == []")

def test_nulllist2_int(config_data):
	check_vsql(config_data, "[] * 2 == []")

def test_nulllist3_int(config_data):
	check_vsql(config_data, "[None, None] * 0 == []")

def test_nulllist4_int(config_data):
	check_vsql(config_data, "[None, None] * 2 == [None, None, None, None]")



"""
Tests for the vSQL slice operator ``A[B:C]``.

The test are done via the Python DB interface.

To run the tests, :mod:`pytest` is required.
"""

from conftest import *


###
### Tests
###

d1 = "@(2000-02-29)"
d2 = "@(2000-03-01)"
d3 = "@(2000-03-02)"
d4 = "@(2000-03-03)"

dt1 = "@(2000-02-29T12:34:56)"
dt2 = "@(2000-03-01T12:34:56)"
dt3 = "@(2000-03-02T12:34:56)"
dt4 = "@(2000-03-03T12:34:56)"

def test_str_1(config_data):
	check_vsql(config_data, "app.p_str_value.value[1:3] == 'ur'")

def test_str_2(config_data):
	check_vsql(config_data, "app.p_str_value.value[-3:-1] == 'ur'")

def test_str_3(config_data):
	check_vsql(config_data, "app.p_str_value.value[4:10] == ''")

def test_str_4(config_data):
	check_vsql(config_data, "app.p_str_value.value[-10:-5] == ''")

def test_str_5(config_data):
	check_vsql(config_data, "app.p_str_value.value[1:] == 'urk'")

def test_str_6(config_data):
	check_vsql(config_data, "app.p_str_value.value[-3:] == 'urk'")

def test_str_7(config_data):
	check_vsql(config_data, "app.p_str_value.value[4:] == ''")

def test_str_8(config_data):
	check_vsql(config_data, "app.p_str_value.value[-10:] == 'gurk'")

def test_str_9(config_data):
	check_vsql(config_data, "app.p_str_value.value[:3] == 'gur'")

def test_str_10(config_data):
	check_vsql(config_data, "app.p_str_value.value[:-1] == 'gur'")

def test_str_11(config_data):
	check_vsql(config_data, "app.p_str_value.value[:10] == 'gurk'")

def test_str_12(config_data):
	check_vsql(config_data, "app.p_str_value.value[:-5] == ''")

def test_str_13(config_data):
	check_vsql(config_data, "app.p_str_value.value[:] == 'gurk'")

def test_str_14(config_data):
	check_vsql(config_data, "app.p_str_value.value[None:None] == 'gurk'")

def test_intlist_1(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][1:3] == [2, 3]")

def test_intlist_2(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][-3:-1] == [2, 3]")

def test_intlist_3(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][4:10] == []")

def test_intlist_4(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][-10:-5] == []")

def test_intlist_5(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][1:] == [2, 3, 4]")

def test_intlist_6(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][-3:] == [2, 3, 4]")

def test_intlist_7(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][4:] == []")

def test_intlist_8(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][-10:] == [1, 2, 3, 4]")

def test_intlist_9(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][:3] == [1, 2, 3]")

def test_intlist_10(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][:-1] == [1, 2, 3]")

def test_intlist_11(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][:10] == [1, 2, 3, 4]")

def test_intlist_12(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][:-5] == []")

def test_intlist_13(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][:] == [1, 2, 3, 4]")

def test_intlist_14(config_data):
	check_vsql(config_data, "[1, 2, 3, 4][None:None] == [1, 2, 3, 4]")

def test_numberlist_1(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][1:3] == [2.2, 3.3]")

def test_numberlist_2(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][-3:-1] == [2.2, 3.3]")

def test_numberlist_3(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][4:10] == []")

def test_numberlist_4(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][-10:-5] == []")

def test_numberlist_5(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][1:] == [2.2, 3.3, 4.4]")

def test_numberlist_6(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][-3:] == [2.2, 3.3, 4.4]")

def test_numberlist_7(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][4:] == []")

def test_numberlist_8(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][-10:] == [1.1, 2.2, 3.3, 4.4]")

def test_numberlist_9(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][:3] == [1.1, 2.2, 3.3]")

def test_numberlist_10(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][:-1] == [1.1, 2.2, 3.3]")

def test_numberlist_11(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][:10] == [1.1, 2.2, 3.3, 4.4]")

def test_numberlist_12(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][:-5] == []")

def test_numberlist_13(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][:] == [1.1, 2.2, 3.3, 4.4]")

def test_numberlist_14(config_data):
	check_vsql(config_data, "[1.1, 2.2, 3.3, 4.4][None:None] == [1.1, 2.2, 3.3, 4.4]")

def test_datelist_1(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][1:3] == [{d2}, {d3}]")

def test_datelist_2(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][-3:-1] == [{d2}, {d3}]")

def test_datelist_3(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][4:10] == []")

def test_datelist_4(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][-10:-5] == []")

def test_datelist_5(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][1:] == [{d2}, {d3}, {d4}]")

def test_datelist_6(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][-3:] == [{d2}, {d3}, {d4}]")

def test_datelist_7(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][4:] == []")

def test_datelist_8(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][-10:] == [{d1}, {d2}, {d3}, {d4}]")

def test_datelist_9(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][:3] == [{d1}, {d2}, {d3}]")

def test_datelist_10(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][:-1] == [{d1}, {d2}, {d3}]")

def test_datelist_11(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][:10] == [{d1}, {d2}, {d3}, {d4}]")

def test_datelist_12(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][:-5] == []")

def test_datelist_13(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][:] == [{d1}, {d2}, {d3}, {d4}]")

def test_datelist_14(config_data):
	check_vsql(config_data, f"[{d1}, {d2}, {d3}, {d4}][None:None] == [{d1}, {d2}, {d3}, {d4}]")

def test_datetimelist_1(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][1:3] == [{dt2}, {dt3}]")

def test_datetimelist_2(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][-3:-1] == [{dt2}, {dt3}]")

def test_datetimelist_3(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][4:10] == []")

def test_datetimelist_4(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][-10:-5] == []")

def test_datetimelist_5(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][1:] == [{dt2}, {dt3}, {dt4}]")

def test_datetimelist_6(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][-3:] == [{dt2}, {dt3}, {dt4}]")

def test_datetimelist_7(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][4:] == []")

def test_datetimelist_8(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][-10:] == [{dt1}, {dt2}, {dt3}, {dt4}]")

def test_datetimelist_9(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][:3] == [{dt1}, {dt2}, {dt3}]")

def test_datetimelist_10(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][:-1] == [{dt1}, {dt2}, {dt3}]")

def test_datetimelist_11(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][:10] == [{dt1}, {dt2}, {dt3}, {dt4}]")

def test_datetimelist_12(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][:-5] == []")

def test_datetimelist_13(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][:] == [{dt1}, {dt2}, {dt3}, {dt4}]")

def test_datetimelist_14(config_data):
	check_vsql(config_data, f"[{dt1}, {dt2}, {dt3}, {dt4}][None:None] == [{dt1}, {dt2}, {dt3}, {dt4}]")

def test_nulllist_1(config_data):
	check_vsql(config_data, f"[None, None, None, None][1:3] == [None, None]")

def test_nulllist_2(config_data):
	check_vsql(config_data, f"[None, None, None, None][-3:-1] == [None, None]")

def test_nulllist_3(config_data):
	check_vsql(config_data, f"[None, None, None, None][4:10] == []")

def test_nulllist_4(config_data):
	check_vsql(config_data, f"[None, None, None, None][-10:-5] == []")

def test_nulllist_5(config_data):
	check_vsql(config_data, f"[None, None, None, None][1:] == [None, None, None]")

def test_nulllist_6(config_data):
	check_vsql(config_data, f"[None, None, None, None][-3:] == [None, None, None]")

def test_nulllist_7(config_data):
	check_vsql(config_data, f"[None, None, None, None][4:] == []")

def test_nulllist_8(config_data):
	check_vsql(config_data, f"[None, None, None, None][-10:] == [None, None, None, None]")

def test_nulllist_9(config_data):
	check_vsql(config_data, f"[None, None, None, None][:3] == [None, None, None]")

def test_nulllist_10(config_data):
	check_vsql(config_data, f"[None, None, None, None][:-1] == [None, None, None]")

def test_nulllist_11(config_data):
	check_vsql(config_data, f"[None, None, None, None][:10] == [None, None, None, None]")

def test_nulllist_12(config_data):
	check_vsql(config_data, f"[None, None, None, None][:-5] == []")

def test_nulllist_13(config_data):
	check_vsql(config_data, f"[None, None, None, None][:] == [None, None, None, None]")

def test_nulllist_14(config_data):
	check_vsql(config_data, f"[None, None, None, None][None:None] == [None, None, None, None]")

def test_nulllist_15(config_data):
	check_vsql(config_data, f"[][:] == []")
